package com.example.bradysdk.printercommunication;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import android.Manifest;
import android.content.Context;

import androidx.test.rule.GrantPermissionRule;

import com.bradysdk.api.printerdiscovery.DiscoveredPrinterInformation;
import com.bradysdk.api.printerdiscovery.PrinterDiscovery;
import com.bradysdk.api.printerdiscovery.PrinterDiscoveryListener;
import com.bradysdk.printengine.printinginterface.PrinterDiscoveryFactory;

import org.junit.After;
import org.junit.Before;
import org.junit.Rule;
import org.junit.Test;

import java.util.ArrayList;
import java.util.List;

/**
 * These tests are located in the app section of the project because
 * printer discovery/communication requires an app context which an sdk doesn't have
 */
public class BluetoothPrinterDiscoveryTests implements PrinterDiscoveryListener {

    @Rule public GrantPermissionRule mRuntimePermissionRuleLocation = GrantPermissionRule.grant(android.Manifest.permission.ACCESS_FINE_LOCATION);
    @Rule public GrantPermissionRule mRuntimePermissionRuleLocation2 = GrantPermissionRule.grant(Manifest.permission.ACCESS_COARSE_LOCATION);
    @Rule public GrantPermissionRule mRuntimePermissionRuleLocation3 = GrantPermissionRule.grant(Manifest.permission.ACCESS_BACKGROUND_LOCATION);

    @Rule public GrantPermissionRule mRuntimePermissionRuleBluetooth = GrantPermissionRule.grant(android.Manifest.permission.BLUETOOTH);
    @Rule public GrantPermissionRule mRuntimePermissionRuleBluetoothAdmin = GrantPermissionRule.grant(android.Manifest.permission.BLUETOOTH_ADMIN);

    List<DiscoveredPrinterInformation> foundPrinters = new ArrayList<>();
    PrinterDiscovery printerDiscovery;
    private final String PRINTER_NAME = "M611-142";

    @Before
    public void bluetoothPrinterDiscoverySetup(){
        foundPrinters = new ArrayList<>();
        Context context = androidx.test.core.app.ApplicationProvider.getApplicationContext();
        List<PrinterDiscoveryListener> printerDiscoveryListeners = new ArrayList<>();
        printerDiscoveryListeners.add(this);
        printerDiscovery = PrinterDiscoveryFactory.getPrinterDiscovery(context, printerDiscoveryListeners);
    }

    @After
    public void bluetoothPrinterDiscoveryTeardown(){
        printerDiscovery.stopPrinterDiscovery();
    }

    @Test
    public void test_findBTPrintersConnectedViaPhoneSettings() {

        // Note: go into phone's bluetooth settings and pair to M611 Production3 printer
        try{
            printerDiscovery.startBluetoothPrinterDiscovery(true);

            // wait 10 seconds for the printer to be found
            Thread.sleep(10000);

            boolean m11Production3Found = false;
            for(DiscoveredPrinterInformation printer : foundPrinters) {
                 if(printer.getName().equals(PRINTER_NAME)) {
                     m11Production3Found = true;
                 }
            }

            assertTrue(m11Production3Found);

        } catch (Exception e) {
            fail(e.getMessage());
        }
    }

    @Test
    public void test_noBTPrintersFoundWhenNoneConnectedViaPhoneSettings() {

        // Note: go into phone's bluetooth settings and forget all connected brady printers
        try{
            printerDiscovery.startBluetoothPrinterDiscovery(true);

            // wait 10 seconds for the printer to be found
            Thread.sleep(10000);

            assertEquals(0, foundPrinters.size());

        } catch (Exception e) {
            fail(e.getMessage());
        }
    }

    @Test
    public void test_findBTPrintersViaDiscovery() {

        // Note: be sure at least one M611 printer is turned on nearby but NOT connected (tap 'forget device' on phone's bt settings page)

        try{
            printerDiscovery.startBluetoothPrinterDiscovery(false);

            // wait for the printer to be found
            Thread.sleep(30000);

            assertTrue(foundPrinters.size() > 0);

        } catch (Exception e) {
            fail(e.getMessage());
        } catch (Throwable throwable) {
            throwable.printStackTrace();
        }
    }

    @Test
    public void test_findSpecificBTPrinterViaDiscovery() {

        // Note: be sure M611-142 printer is turned on nearby but NOT connected (tap 'forget device' on phone's bt settings page)

        try{
            printerDiscovery.startBluetoothPrinterDiscovery(false);

            // wait for the printer to be found
            Thread.sleep(30000);

            boolean m11Production3Found = false;
            for(DiscoveredPrinterInformation printer : foundPrinters) {
                if(printer.getName().equals(PRINTER_NAME)) {
                    m11Production3Found = true;
                    break;
                }
            }
            assertTrue(m11Production3Found);

        } catch (Exception e) {
            fail(e.getMessage());
        } catch (Throwable throwable) {
            throwable.printStackTrace();
        }
    }

    //region overrides of PrinterDiscoveryListener

    @Override
    public void printerDiscovered(DiscoveredPrinterInformation discoveredPrinterInformation) {

        foundPrinters.add(discoveredPrinterInformation);
    }

    @Override
    public void printerDiscoveryStarted() {
    }

    @Override
    public void printerDiscoveryStopped() {
    }

    //endregion overrides of PrinterDiscoveryListener

}
