package com.example.bradysdk.printercommunication;

import static com.bradysdk.api.printerconnection.PrinterProperties.AcConnected;
import static com.bradysdk.api.printerconnection.PrinterProperties.MessageTitle;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertSame;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import android.Manifest;
import android.content.Context;
import android.util.Log;

import androidx.test.rule.GrantPermissionRule;

import com.bradysdk.api.printerconnection.PrinterDetails;
import com.bradysdk.api.printerconnection.PrinterProperties;
import com.bradysdk.api.printerconnection.PrinterStatus;
import com.bradysdk.api.printerconnection.PrinterUpdateListener;
import com.bradysdk.api.printerdiscovery.DiscoveredPrinterInformation;
import com.bradysdk.api.printerdiscovery.PrinterDiscovery;
import com.bradysdk.api.printerdiscovery.PrinterDiscoveryListener;
import com.bradysdk.api.templates.Template;
import com.bradysdk.printengine.printerservices.WifiPrinterServices;
import com.bradysdk.printengine.printinginterface.PrinterDiscoveryFactory;
import com.bradysdk.printengine.templateinterface.TemplateFactory;

import org.junit.After;
import org.junit.Before;
import org.junit.Rule;
import org.junit.Test;

import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

public class WifiPrinterCommunicationTests implements PrinterDiscoveryListener, PrinterUpdateListener {

    @Rule public GrantPermissionRule mRuntimePermissionRuleLocation = GrantPermissionRule.grant(android.Manifest.permission.ACCESS_FINE_LOCATION);
    @Rule public GrantPermissionRule mRuntimePermissionRuleLocation2 = GrantPermissionRule.grant(Manifest.permission.ACCESS_COARSE_LOCATION);
    @Rule public GrantPermissionRule mRuntimePermissionRuleLocation3 = GrantPermissionRule.grant(Manifest.permission.ACCESS_BACKGROUND_LOCATION);

    @Rule public GrantPermissionRule mRuntimePermissionRuleInternet = GrantPermissionRule.grant(android.Manifest.permission.INTERNET);
    @Rule public GrantPermissionRule mRuntimePermissionRuleInternetMultiCast = GrantPermissionRule.grant(android.Manifest.permission.CHANGE_WIFI_MULTICAST_STATE);

    List<DiscoveredPrinterInformation> foundPrinters = new ArrayList<>();
    PrinterDiscovery printerDiscovery;
    private final String PRINTER_NAME = "M611-142";
    private List<PrinterProperties> updatedPrinterProperties = new ArrayList<>();
    private PrinterDetails printerDetails;
    private WifiPrinterServices wifiPrinterServices;

    @Before
    public void wifiPrinterDiscoverySetup(){
        foundPrinters = new ArrayList<>();
        Context context = androidx.test.core.app.ApplicationProvider.getApplicationContext();
        List<PrinterDiscoveryListener> printerDiscoveryListeners = new ArrayList<>();
        printerDiscoveryListeners.add(this);
        printerDiscovery = PrinterDiscoveryFactory.getPrinterDiscovery(context, printerDiscoveryListeners);
        wifiPrinterServices = new WifiPrinterServices(context);
        wifiPrinterServices.addPrinterDiscoveryListeners(printerDiscoveryListeners);

        DiscoveredPrinterInformation printer = discoverPrinter(PRINTER_NAME);

        printerDetails = PrinterDiscoveryFactory.getPrinterDiscovery(context, printerDiscoveryListeners).connectToDiscoveredPrinter(context, printer, Collections.singletonList(this));

        assertNotNull(printerDetails);
    }


    @After
    public void wifiPrinterDiscoveryTeardown(){
        printerDiscovery.stopPrinterDiscovery();
    }

    @Test
    public void test_receiveUpdateFromWifiPrinterWhenLockLeverOpened() {

        // Note: be sure M611-142 printer is turned on nearby 

        try{

            // put a breakpoint and open the printer lock lever
            Thread.sleep(1000);

            assertNotNull(printerDetails);
            assertTrue(updatedPrinterProperties.stream().anyMatch(p -> p == MessageTitle));
            assertEquals("Lock Lever Open", printerDetails.getPrinterStatusMessageTitle());


        } catch (Exception e) {
            fail(e.getMessage());
        } catch (Throwable throwable) {
            throwable.printStackTrace();
        }
    }

    @Test
    public void test_receiveUpdateFromWifiPrinterWhenUnpluggedFromACPower() {

        // Note: be sure M611-142 printer is turned on nearby 

        try{
            assertTrue(printerDetails.getIsAcConnected());

            // put a breakpoint and unplug the printer
            Thread.sleep(1000);

            assertNotNull(printerDetails);
            assertTrue(updatedPrinterProperties.stream().anyMatch(p -> p == AcConnected));
            assertFalse(printerDetails.getIsAcConnected());


        } catch (Exception e) {
            fail(e.getMessage());
        } catch (Throwable throwable) {
            throwable.printStackTrace();
        }
    }


    @Test
    public void test_partInfoFromWifiPrinterIsCorrect() {

        // Note: be sure M611-142 printer is turned on nearby and the correct part is installed

        try{
            assertEquals("M61C-625-422", printerDetails.getSupplyName());
            
        } catch (Exception e) {
            fail(e.getMessage());
        } catch (Throwable throwable) {
            throwable.printStackTrace();
        }
    }

    @Test
    public void test_ribbonInfoFromWifiPrinterIsCorrect() {

        // Note: be sure M611-142 printer is turned on nearby and the correct ribbon is installed

        try{
            assertEquals("R6010", printerDetails.getRibbonName());

        } catch (Exception e) {
            fail(e.getMessage());
        } catch (Throwable throwable) {
            throwable.printStackTrace();
        }
    }

    @Test
    public void test_partMismatchError() {
        Context context = androidx.test.core.app.ApplicationProvider.getApplicationContext();

        // Note: be sure M611-142 printer is turned on nearby and the correct part is installed
        Template template;
        try (InputStream is = this.getClass().getResourceAsStream("/SDKImageTemplate1.BWT")) {

            template = TemplateFactory.getTemplate(is, context);

            assertFalse(printerDetails.checkForPartMismatch(template));
        } catch (
                IOException e) {
            e.printStackTrace();
        } catch (Exception e) {
            fail(e.getMessage());
        } catch (Throwable throwable) {
            throwable.printStackTrace();
        }
    }

    @Test
    public void test_noPartMismatchError() {
        Context context = androidx.test.core.app.ApplicationProvider.getApplicationContext();

        // Note: be sure M611-142 printer is turned on nearby and the correct part is installed
        Template template;
        try (InputStream is = this.getClass().getResourceAsStream("/M61C-625-422Template.BWT")) {

            template = TemplateFactory.getTemplate(is, context);

            assertFalse(printerDetails.checkForPartMismatch(template));
        } catch (
                IOException e) {
            e.printStackTrace();
        } catch (Exception e) {
            fail(e.getMessage());
        } catch (Throwable throwable) {
            throwable.printStackTrace();
        }
    }

    @Test
    public void test_statusAndInfo() {

        // Note: be sure M611-142 printer is turned on nearby and set up correctly
        try  {

            PrinterStatus status = printerDetails.getPrinterStatus();
            assertSame(status, PrinterStatus.Warning);
            String msgTitle = printerDetails.getPrinterStatusMessageTitle();
            assertEquals("Ribbon/Supply Mismatch", msgTitle);
            String msg = printerDetails.getPrinterStatusMessage();
            assertNotNull(msg);
            String remedyExplanationMessage = printerDetails.getPrinterStatusRemedyExplanationMessage();
            assertNotNull(remedyExplanationMessage);

            Integer ribbonRemainingPercentage = printerDetails.getRibbonRemainingPercentage();
            assertNotNull(ribbonRemainingPercentage);
            Integer supplyRemainingPercentage = printerDetails.getSupplyRemainingPercentage();
            assertNotNull(supplyRemainingPercentage);
            Integer batteryLevelPercentage = printerDetails.getBatteryLevelPercentage();
            assertNotNull(batteryLevelPercentage);

        } catch (Exception e) {
            fail(e.getMessage());
        } catch (Throwable throwable) {
            throwable.printStackTrace();
        }
    }

    private DiscoveredPrinterInformation discoverPrinter(String printerName){
        try{
            printerDiscovery.startWifiPrinterDiscovery();

            // wait for the printer to be found
            Thread.sleep(30000);

            for(DiscoveredPrinterInformation printer : foundPrinters) {
                if(printer.getName().equals(printerName)) {
                    return printer;
                }
            }

        } catch (Exception e) {
            fail(e.getMessage());
        } catch (Throwable throwable) {
            throwable.printStackTrace();
        }

        return null;
    }

    //region overrides of PrinterDiscoveryListener

    @Override
    public void printerDiscovered(DiscoveredPrinterInformation discoveredPrinterInformation) {

        foundPrinters.add(discoveredPrinterInformation);
    }

    @Override
    public void printerDiscoveryStarted() {
    }

    @Override
    public void printerDiscoveryStopped() {
    }

    //endregion overrides of PrinterDiscoveryListener

    //region Overrides of PrinterUpdateListener

    @Override
    public void PrinterUpdate(List<PrinterProperties> list) {

        Log.d("~~~~~Step 7 (wifi)", "PrinterUpdate");

        updatedPrinterProperties = list;

    }

    //endregion Overrides of PrinterUpdateListener

}
