package com.example.bradysdk;

import android.content.Context;
import android.graphics.Bitmap;
import android.util.Log;
import android.util.TypedValue;

import com.bradysdk.api.printerconnection.CutOption;
import com.bradysdk.api.printerconnection.PrinterDetails;
import com.bradysdk.api.printerconnection.PrinterUpdateListener;
import com.bradysdk.api.printerconnection.PrintingOptions;
import com.bradysdk.api.printerdiscovery.DiscoveredPrinterInformation;
import com.bradysdk.api.printerdiscovery.PrinterDiscovery;
import com.bradysdk.api.printerdiscovery.PrinterDiscoveryListener;
import com.bradysdk.api.templates.Template;
import com.bradysdk.printengine.printerservices.BleGenericPrinterInformation;
import com.bradysdk.printengine.printerservices.BluetoothPrinterInformation;
import com.bradysdk.printengine.printerservices.WifiPrinterInformation;
import com.bradysdk.printengine.printinginterface.PrinterDiscoveryFactory;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * Handles all front end logic controlled by the MainActivity
 */
public class MainPresenter implements PrinterDiscoveryListener {

    final private MainInterface i;
    final private Context context;
    private PrinterDiscovery printerDiscovery;
    private PrinterDetails printerDetails;
    private final List<DiscoveredPrinterInformation> foundPrinters = new ArrayList();

    public MainPresenter(com.example.bradysdk.MainInterface i, com.example.bradysdk.MainActivity context){
        this.i = i;
        this.context = context;
    }

    /**
     * Starts scanning for printers in the area through wifi, bluetooth classic, and bluetooth low energy
     */
    public void setupPrinterDiscovery() {

        try {
            List<PrinterDiscoveryListener> printerDiscoveryListeners = new ArrayList<>();
            printerDiscoveryListeners.add(this);
            printerDiscovery = PrinterDiscoveryFactory.getPrinterDiscovery(context.getApplicationContext(), printerDiscoveryListeners);
            i.sendPrintDiscovery(printerDiscovery);

            DiscoveredPrinterInformation lastConnectedPrinter = printerDiscovery.getLastConnectedPrinter();
            if (lastConnectedPrinter != null && !lastConnectedPrinter.getName().equals("")) {
                try {
                    connectToPrinter(lastConnectedPrinter, (PrinterUpdateListener) context, true);
                }
                catch(Exception e) {
                    i.showToast("Make sure the printer is on and in range!");
                }
            } else {
                printerDiscovery.startBlePrinterDiscovery();
                printerDiscovery.startWifiPrinterDiscovery();
                //printerDiscovery.startBluetoothPrinterDiscovery(false);
            }

        } catch (Exception ex) {
            System.out.println(ex.getMessage());
        }
    }

    /**
     * Attempts to connect to the discovered printer
     * @param pul the printer listener
     * @param showDetailPage boolean that determines if the details page is shown to the user
     */
    public void connectToPrinter(DiscoveredPrinterInformation printerSelected, PrinterUpdateListener pul, boolean showDetailPage) {
        Runnable r = () -> {
            try {
                i.showProgress();
                printerDetails = printerDiscovery.connectToDiscoveredPrinter(context, printerSelected, Collections.singletonList(pul));
                if (printerDiscovery.getHaveOwnership() != null) {
                    try {
                        if (printerDetails == null && !printerDiscovery.getHaveOwnership() && printerSelected.getName().contains("M211") && printerDiscovery.getLastConnectedPrinter().getName().contains("M211")) {
                            i.showToast("Hold Power Button To Reclaim Ownership!");
                        } else if (printerDetails == null && !printerDiscovery.getHaveOwnership()) {
                            i.showToast("You must properly disconnect from your previous printer to gain ownership of the new one.");
                        }
                    } catch (NullPointerException ex) {
                    }
                } else {
                }
            }
            catch(Exception e) {
                i.showToast("Make sure the printer is on and in range.");
            }
            i.setPrinterDetails(printerDetails, showDetailPage);
            i.hideProgress();
        };
        Thread connectThread = new Thread(r);
        connectThread.start();
    }

    /**
     * Parses and then sends the Template name as a string to the MainActivity
     * @param id of the template
     */
    public void getTemplate(int id) {
        TypedValue value = new TypedValue();
        context.getResources().getValue(id, value, true);
        String[] nameSplit = value.string.toString().split("/");
        i.getTemplate(nameSplit[nameSplit.length-1]);
    }

    /**
     * Attempts to print the template using the Printer Details and the current connection
     * @param template what will print
     */
    public void print(Template template) {
        PrintingOptions printingOptions = new PrintingOptions();
        printingOptions.setCutOption(CutOption.EndOfJob);
        printingOptions.setNumberOfCopies(1);
        Runnable r = () -> {
            i.showProgress();
            try {
                printerDetails.print(context, template, printingOptions, null);
            }
            catch(Exception e) {
                e.printStackTrace();
            }
            i.hideProgress();
        };
        Thread printThread = new Thread(r);
        printThread.start();
    }

    public void print(Bitmap bitmap) {
        PrintingOptions printingOptions = new PrintingOptions();
        printingOptions.setCutOption(CutOption.EndOfJob);
        printingOptions.setNumberOfCopies(1);
        Runnable r = () -> {
            i.showProgress();
            printerDetails.print(context, bitmap, printingOptions, false);
            i.hideProgress();
        };
        Thread printThread = new Thread(r);
        printThread.start();
    }

    public List<DiscoveredPrinterInformation> getDiscoveredPrinterNames(){
        if(printerDiscovery != null) {
            return foundPrinters;
        }
        else{
            return null;
        }
    }

    public void stopDiscovery() {
        printerDiscovery.stopPrinterDiscovery();
    }

    public void cutLabel() {
        printerDetails.cutLabel();
    }

    public void feed() {
        printerDetails.feed();
    }

    public void turnOff() {
        printerDetails.turnOff(30);
    }

    @Override
    public void printerDiscovered(DiscoveredPrinterInformation discoveredPrinterInformation) {
        if(discoveredPrinterInformation instanceof BleGenericPrinterInformation){
            if(printerDoesNotExist(discoveredPrinterInformation)){
                foundPrinters.add(discoveredPrinterInformation);
            }
        }
        else if(discoveredPrinterInformation instanceof BluetoothPrinterInformation){
            if(printerDoesNotExist(discoveredPrinterInformation)){
                foundPrinters.add(discoveredPrinterInformation);
            }
        }
        else{
            if(printerDoesNotExist(discoveredPrinterInformation)){
                foundPrinters.add(discoveredPrinterInformation);
            }
        }
    }

    private boolean printerDoesNotExist(DiscoveredPrinterInformation discoveredPrinter) {
        boolean wasFound = false;
        for(DiscoveredPrinterInformation printer : foundPrinters){
            if (printer.getName().equals(discoveredPrinter.getName())) {
                if (printer instanceof BleGenericPrinterInformation && discoveredPrinter instanceof BleGenericPrinterInformation) {
                    wasFound = true;
                    break;
                } else if (printer instanceof BluetoothPrinterInformation && discoveredPrinter instanceof BluetoothPrinterInformation) {
                    wasFound = true;
                    break;
                } else if (printer instanceof WifiPrinterInformation && discoveredPrinter instanceof WifiPrinterInformation) {
                    wasFound = true;
                    break;
                }
            }
        }
        return !wasFound;
    }

    @Override
    public void printerDiscoveryStarted() {
        Log.i("Discovery Started", "The Printer Discovery scan was started!");
    }

    @Override
    public void printerDiscoveryStopped() {
        Log.i("Discovery Stopped", "The Printer Discovery scan was stopped!");
    }

    public void clearPrinterDetails() {
        this.printerDetails = null;
    }
}
